@@ -125,26 +125,15 @@ use generic_array::{GenericArray, typenum::consts::U20};
 /// The digest output.
 pub type Output = GenericArray<u8, U20>;
 
-#[cfg(feature = "digest-trait")]
-use digest::consts::U64;
 #[cfg(feature = "digest-trait")]
 pub use digest::{self, Digest};
 #[cfg(feature = "digest-trait")]
-use digest::{BlockInput, FixedOutputDirty, Reset, Update};
+use digest::{FixedOutput, HashMarker, OutputSizeUser, Reset, Update};
 
 /// Configures the collision-detecting SHA-1 algorithm.
+#[derive(Default)]
 pub struct Builder(sys::SHA1_CTX);
 
-impl Default for Builder {
-    fn default() -> Self {
-        Self(unsafe {
-            let mut ctx = core::mem::MaybeUninit::uninit();
-            sys::SHA1DCInit(ctx.as_mut_ptr());
-            ctx.assume_init()
-        })
-    }
-}
-
 impl Builder {
     /// Configures collision mitigation.
     ///
@@ -215,13 +204,11 @@ impl Default for Sha1CD {
 }
 
 #[cfg(feature = "digest-trait")]
-impl BlockInput for Sha1CD {
-    type BlockSize = U64;
-}
+impl HashMarker for Sha1CD {}
 
 #[cfg(feature = "digest-trait")]
 impl Update for Sha1CD {
-    fn update(&mut self, input: impl AsRef<[u8]>) {
+    fn update(&mut self, input: &[u8]) {
         Sha1CD::update(self, input);
     }
 }
@@ -230,31 +217,30 @@ impl Update for Sha1CD {
 impl Reset for Sha1CD {
     fn reset(&mut self) {
         Sha1CD::reset(self);
-        let safe_hash = self.0.safe_hash;
-        let ubc_check = self.0.ubc_check;
-        let detect_coll = self.0.detect_coll;
-        let reduced_round_coll = self.0.reduced_round_coll;
-        let callback = self.0.callback;
-        unsafe {
-            sys::SHA1DCInit(&mut self.0);
-        }
-        self.0.safe_hash = safe_hash;
-        self.0.ubc_check = ubc_check;
-        self.0.detect_coll = detect_coll;
-        self.0.reduced_round_coll = reduced_round_coll;
-        self.0.callback = callback;
     }
 }
 
 #[cfg(feature = "digest-trait")]
-impl FixedOutputDirty for Sha1CD {
+impl OutputSizeUser for Sha1CD {
     type OutputSize = U20;
+}
 
-    fn finalize_into_dirty(&mut self, out: &mut digest::Output<Self>) {
+#[cfg(feature = "digest-trait")]
+impl FixedOutput for Sha1CD {
+    fn finalize_into(mut self, out: &mut Output) {
         let _ = self.finalize_into_dirty_cd(out);
     }
 }
 
+#[cfg(feature = "oid")]
+use const_oid::{AssociatedOid, ObjectIdentifier};
+
+#[cfg(feature = "oid")]
+impl AssociatedOid for Sha1CD {
+    const OID: ObjectIdentifier =
+        ObjectIdentifier::new_unwrap("1.3.14.3.2.26");
+}
+
 impl Sha1CD {
     /// Configures the algorithm.
     pub fn configure() -> Builder {
@@ -281,9 +267,7 @@ impl Sha1CD {
         let detect_coll = self.0.detect_coll;
         let reduced_round_coll = self.0.reduced_round_coll;
         let callback = self.0.callback;
-        unsafe {
-            sys::SHA1DCInit(&mut self.0);
-        }
+        self.0.reset();
         self.0.safe_hash = safe_hash;
         self.0.ubc_check = ubc_check;
         self.0.detect_coll = detect_coll;
